      subroutine isaxpy3d ( ni, nj, nk,  nvirtual, ishift, jshift,
     +                      kshift, vecy, intn, l, facs, vecx, iarea,
     +                      ichoisdis )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 2.0    date 22-03-1996 Marcel: First order upwind scheme
c        version 1.0    date 02-06-1995 (copy of isaxpy2d
c                                      of Guus Segal )
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Perform a vector update for ISNaS over a 3-dimensional region
c    The vector to be added may be shifted with respect to the update vector
c    The update looks as follows:
c    y(i,j,k) = y(i,j,k) + a * x(i+ishift,j+jshift,k+kshift)
c    Both arrays are defined over the complete region, however y is
c    restricted to the region without virtual points and x uses virtual
c    points.
c    In order to get an optimal code the shift has been applied in the
c    declaration instead of the code, so that the code can use the same
c    indices for the vector to be updated as the vector to be added.
c    Since this is dangerous trick the user must be aware of this application
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, ishift, jshift, kshift,
     +        iarea(1:2,1:3), intn, l, ichoisdis
      double precision vecy(ni+1,nj+1,nk+1), facs,
     +                 vecx(1-nvirtual-ishift:ni+nvirtual+1-ishift,
     +                      1-nvirtual-jshift:nj+nvirtual+1-jshift,
     +                      1-nvirtual-kshift:nk+nvirtual+1-kshift)

c     facs      i    Multiplication factor
c     iarea     i    Area of arrays
c     ichoisdis i    Choice parameter with respect to the convection scheme
c                    for discretization of convective term of momentum and
c                    transport equations.
c                    Meaning of ichoisdis:
c                      ichoisdis =  0: Second order central scheme
c                      ichoisdis =  1: First order upwind scheme
c                      ichoisdis =  2: Hybrid central/upwind scheme
c                      ichoisdis =  3: Higher order upwind scheme (kappa-scheme)
c                      ichoisdis =  4: Sweby Phi-limiter
c                      ichoisdis =  5: R-kappa limiter
c                      ichoisdis =  6: MR-kappa limiter
c                      ichoisdis =  7: Symmetric Ratio limiter
c                      ichoisdis =  8: PL-kappa limiter
c                      ichoisdis =  9: MPL1-kappa limiter
c                      ichoisdis = 10: MPL2-kappa limiter
c                      ichoisdis = 11: Symmetric PL-kappa limiter
c     intn      i    Integration point number
c     ishift    i    Shift in i-direction
c     jshift    i    Shift in j-direction
c     kshift    i    Shift in k-direction
c     l         i    General loop variable
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     vecx      i    Vector to be added
c     vecy     i/o   Vector to be updated
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k

c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      if ( ichoisdis.eq.0 ) then

c     --- central scheme

          do 120 k = iarea(1,3), iarea(2,3)
            do 110 j = iarea(1,2), iarea(2,2)
              do 100 i = iarea(1,1), iarea(2,1)
                vecy(i,j,k) = vecy(i,j,k)+facs*vecx(i,j,k)
100           continue
110         continue
120       continue

      else

c     --- first order or higher order upwind scheme

        if ( intn.eq.3 .or. intn.eq.5 .or. intn.eq.7 ) then

c       --- integration points  3, 5, 7

          if ( l.eq.1 ) then

            do 220 k = iarea(1,3), iarea(2,3)
              do 210 j = iarea(1,2), iarea(2,2)
                do 200 i = iarea(1,1), iarea(2,1)
                  vecy(i,j,k) = vecy(i,j,k)+
     +                 5d-1*(1d0+sign(1d0,vecx(i,j,k)))*vecx(i,j,k)
200             continue
210           continue
220         continue

          else if ( l.eq.2 ) then

            do 320 k = iarea(1,3), iarea(2,3)
              do 310 j = iarea(1,2), iarea(2,2)
                do 300 i = iarea(1,1), iarea(2,1)
                  vecy(i,j,k) = vecy(i,j,k)+
     +                 5d-1*(1d0-sign(1d0,vecx(i,j,k)))*vecx(i,j,k)
300             continue
310           continue
320         continue

          end if

        else if ( intn.eq.2 .or. intn.eq.4 .or. intn.eq.6 ) then

c       --- integration points  2, 4, 6

          if ( l.eq.1 ) then

            do 420 k = iarea(1,3), iarea(2,3)
              do 410 j = iarea(1,2), iarea(2,2)
                do 400 i = iarea(1,1), iarea(2,1)
                  vecy(i,j,k) = vecy(i,j,k)+
     +                 5d-1*(1d0-sign(1d0,vecx(i,j,k)))*vecx(i,j,k)
400             continue
410           continue
420         continue

          else if ( l.eq.2 ) then

            do 520 k = iarea(1,3), iarea(2,3)
              do 510 j = iarea(1,2), iarea(2,2)
                do 500 i = iarea(1,1), iarea(2,1)
                  vecy(i,j,k) = vecy(i,j,k)+
     +                 5d-1*(1d0+sign(1d0,vecx(i,j,k)))*vecx(i,j,k)
500             continue
510           continue
520         continue

          end if

        else

c       --- integration point 1

          do 620 k = iarea(1,3), iarea(2,3)
            do 610 j = iarea(1,2), iarea(2,2)
              do 600 i = iarea(1,1), iarea(2,1)
                vecy(i,j,k) = vecy(i,j,k)+facs*vecx(i,j,k)
600           continue
610         continue
620       continue

        end if

      end if

      end
