      subroutine iscomlr2 (ni, nj, ndegfd, nvirtual, solut, eddy,
     +                     rhotild, shiftr, shiftrfo )
c ======================================================================
c
c        programmer     Duncan R. Vanderheul
c
c        version 1.1    date 01-04-1999   Duncan: total revision
c        version 1.0    date 15-07-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   ISNaS compressible subroutine for computation of density-biased
c   upwinding in pressure correction equation with limiter function
c **********************************************************************
c
c                       KEYWORDS
c
c     compressible
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, ndegfd, nvirtual
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                 1-nvirtual:nj+1+nvirtual,ndegfd,*),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                 1-nvirtual:nj+1+nvirtual,2),
     +                 rhotild(ni+1,nj+1,*),
     +                 shiftr(ni+1,nj+1,*),
     +                 shiftrfo(ni+1,nj+1,*)

c     eddy      i    reserved
c     ndegfd    i    Number of degrees of freedom
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhotild   i    Nondimensional value of the upwinded density at the face
c     shiftr    i    Shift variable used in compressible density upwinding.
c     shiftrfo  i    Shift variable used in compressible density upwinding,
c                    first order.
c     solut     i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j
      double precision   uvel, vvel

c     i         General loop variable
c     j         General loop variable
c     uvel      Velocity in i direction
c     vvel      Velocity in j direction
c **********************************************************************
c
c                       I/O
c
c     none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      double precision isdlimt

c     ISDLIMT   limiter function for the upwind density bias
c     ISSETCON  Make array equal to a constant
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     none
c **********************************************************************
c
c                       PSEUDO CODE
c
c  The continuity equation is regarded as a convection equation for the
c  density, where the convective terms are discretised with a limited kappa
c  scheme. For now the ISNAS limited higher order upwind scheme is
c  implemented.
c
c  The ISNAS higher order upwind scheme is implemented in a deferred
c  correction manner:
c  the fluxes at the faces are approximated as:
c                      LO n+1           HO n          LO n
c  rho_i+1/2=( rho_i+1/2 )  + ((rho_i+1/2 )- (rho_i+1/2 ))
c
c  where LO refers to the first order upwind scheme
c  and HO to the higher order limited scheme
c  in effect this means that the density upwind contributions to the
c  matrix are those obtained with the first order upwind scheme, but that
c  in the right hand side:
c                    ~
c                   rho_i+1/2
c            ---------------------
c            1/2(rho_i + rho_i+1)
c
c
c                    ~
c                   rho_i+1/2 is discretised in an higher order limited
c                   upwind manner.
c
c  - reset the value of array switch, containing the value of the limiter
c
c  for all points do
c     compute the value of rhotild, the limiter function argument
c                in i direction, and the value of the limter
c  for all points do
c     compute the value of rhotild, the limiter function argument
c                in j direction, and the value of the limter
c  for all points do                             ~
c     compute the value of the upwinded density rho in i direction
c  for all points do
c     compute the value of the upwinded density rho in j direction
c
c  - reset the value of array switch, containing the value of the limiter
c    to the value for full density bias for computation of the matrix
c    contributions
c
c
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      call issetcon(rhotild,2*(ni+1)*(nj+1),0d0)

c     --- computation of the value of rhotild in i direction

      do j=1,nj+1
         do i=1,ni+1

            if( abs(eddy(i+1,j,2)-eddy(i,j,2)).gt.1d-12) then
               rhotild(i,j,1)=(eddy(i,j,2)-eddy(i-1,j,2))/
     +                (eddy(i+1,j,2)-eddy(i,j,2))
            else
               rhotild(i,j,1)=(eddy(i,j,2)-eddy(i-1,j,2))*
     +                 sign(1d12,(eddy(i+1,j,2)-eddy(i,j,2)))
            end if

            if( abs(rhotild(i,j,1)).lt.1d-12)
     +           rhotild(i,j,1)=rhotild(i,j,1)+1d-12
         end do
      end do

c     --- computation of the value of rhotild in j direction

      do j=1,nj+1
         do i=1,ni+1

            if( abs(eddy(i,j+1,2)-eddy(i,j,2)).gt.1d-12) then
               rhotild(i,j,2)=(eddy(i,j,2)-eddy(i,j-1,2))/
     +                (eddy(i,j+1,2)-eddy(i,j,2))
            else
               rhotild(i,j,2)=(eddy(i,j,2)-eddy(i,j-1,2))*
     +                 sign(1d12,(eddy(i,j+1,2)-eddy(i,j,2)))
            end if

            if( abs(rhotild(i,j,2)).lt.1d-12)
     +           rhotild(i,j,2)=rhotild(i,j,2)+1d-12
         end do
      end do

c     --- computation of higher order shifted density Sro/ro in points (i-1/2,j)

      i=1
      do j=1,nj+1
        shiftr(i,j,1)=1d0
      end do

      do i = 1, ni+1
         do j = 1, nj+1
               uvel=sign(1d0,solut(i,j,1,1))
               shiftr(i,j,1) =(0.5d0*(1d0+uvel)*
     +                           (eddy(i-1,j,2)+
     +                             0.5d0*isdlimt(rhotild(i-1,j,1))*
     +                                (eddy(i,j,2)-eddy(i-1,j,2)))+
     +                         0.5d0*(1d0-uvel)*
     +                           (eddy(i,j,2)+
     +                             0.5d0*isdlimt(1d0/rhotild(i,j,1))*
     +                                (eddy(i-1,j,2)-eddy(i,j,2))))/
     +                            (0.5d0*(eddy(i-1,j,2)+eddy(i,j,2)))
         end do
      end do

c     --- computation of higher order shifted density Sro/ro in points (i,j-1/2)

      j=1
      do i=1,ni+1
        shiftr(i,j,2)=1d0
      end do

      do i = 1, ni+1
         do j = 2, nj+1
               vvel=sign(1d0,solut(i,j,2,1))
               shiftr(i,j,2) =(0.5d0*(1d0+vvel)*
     +                           (eddy(i,j-1,2)+
     +                             0.5d0*isdlimt(rhotild(i,j-1,2))*
     +                                (eddy(i,j,2)-eddy(i,j-1,2)))+
     +                         0.5d0*(1d0-vvel)*
     +                           (eddy(i,j,2)+
     +                         0.5d0*isdlimt(1d0/rhotild(i,j,2))*
     +                                (eddy(i,j-1,2)-eddy(i,j,2))))/
     +                            (0.5d0*(eddy(i,j-1,2)+eddy(i,j,2)))
         end do
      end do

c     --- computation of first order shifted density Sro/ro in points (i-1/2,j)

      i=1
      do j=1,nj+1
        shiftr(i,j,1)=1d0
      end do

      do i = 1, ni+1
         do j = 1, nj+1
               uvel=sign(1d0,solut(i,j,1,1))
               shiftrfo(i,j,1) =(0.5d0*(1d0+uvel)*
     +                           (eddy(i-1,j,2))+
     +                         0.5d0*(1d0-uvel)*
     +                           (eddy(i,j,2)))/
     +                            (0.5d0*(eddy(i-1,j,2)+eddy(i,j,2)))
         end do
      end do

c     --- computation of first order shifted density Sro/ro in points (i,j-1/2)

      j=1
      do i=1,ni+1
        shiftr(i,j,2)=1d0
      end do

      do i = 1, ni+1
         do j = 2, nj+1
               vvel=sign(1d0,solut(i,j,2,1))
               shiftrfo(i,j,2) =(0.5d0*(1d0+vvel)*
     +                           (eddy(i,j-1,2))+
     +                         0.5d0*(1d0-vvel)*
     +                           (eddy(i,j,2)))/
     +                            (0.5d0*(eddy(i,j-1,2)+eddy(i,j,2)))
         end do
      end do

      end
