      subroutine isdisct2 ( alphad, ni, nj, nvirtual, matrix, tsum,
     +                      veloc, mct, iarea, dcell, cstar, kdif,
     +                      jac, ichoisdis, idiffus )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 2.0    date   17-10-1996 Extra parameter idiffus
c        version 1.0    date   30-10-1995 Subroutine extracted from isdcuam
c
c
c   copyright (c) 1995-1996  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Distribute convective terms for transport equations over the matrix in R2
c    Both central schemes and upwind schemes are allowed
c **********************************************************************
c
c                       KEYWORDS
c
c     discretization
c     transport_matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, iarea(2,2), dcell(2), alphad, ichoisdis,
     +        mct(-2:2,-2:2,-2:2), idiffus
      double precision cstar(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual),
     +                 matrix(ni+1,nj+1,*),
     +                 veloc(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual),
     +                 jac(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual),
     +                 tsum(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual),
     +                 kdif(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,*)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     cstar     i    The capacity in a given integration point
c     dcell     i    reserved
c     iarea     i    Area of arrays
c     ichoisdis i    Choice parameter with respect to the convection scheme
c                    for discretization of convective term of momentum and
c                    transport equations.
c                    Meaning of ichoisdis:
c                      ichoisdis =  0: Second order central scheme
c                      ichoisdis =  1: First order upwind scheme
c                      ichoisdis =  2: Hybrid central/upwind scheme
c                      ichoisdis =  3: Higher order upwind scheme (kappa-scheme)
c                      ichoisdis =  4: Sweby Phi-limiter
c                      ichoisdis =  5: R-kappa limiter
c                      ichoisdis =  6: MR-kappa limiter
c                      ichoisdis =  7: Symmetric Ratio limiter
c                      ichoisdis =  8: PL-kappa limiter
c                      ichoisdis =  9: MPL1-kappa limiter
c                      ichoisdis = 10: MPL2-kappa limiter
c                      ichoisdis = 11: Symmetric PL-kappa limiter
c     idiffus   i    Indicates if the diffusion term must be skipped (1)
c                    or not (0)
c     jac       i    sqrt(g) in pnttyp
c     kdif      i    The diffusion coefficient in a given integration point
c     matrix    o    reserved
c     mct       i    Integer matrix information
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     tsum      i    Flux contributions to the matrix
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer shifti, shiftj, ip, jp, iptensor1,
     +        dx(2), ofstnr(2)
      double precision flux, pe, swfunc

c     dx        Point location in cell relative to local
c               coordinates with (0,0) and (2,2) or
c               (0,0,0) and (2,2,2) as opposite cell corners
c     flux      Contravariant flux component
c     ip        Local counting variable i-direction
c     iptensor1 Tensor index
c     jp        Local counting variable j-direction
c     ofstnr    Offset diagonal number of matrix element that
c               corresponds to the current point of the interpolation
c               or extrapolation formula
c     pe        Mesh-Peclet number
c     shifti    Shift to be added to integration cell index to
c               arrive at the update for that cell i-direction
c     shiftj    Shift to be added to integration cell index to
c               arrive at the update for that cell j-direction
c     swfunc    The switching function which takes care of the selection
c               between the central and standard upwind scheme
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    The distribution of the convective terms in the central scheme is
c    straightforward:
c
c    Consider a transport cell with surrounding cells:
c
c            -------------------------------------
c            |           |           |           |
c            |     +     |     +     |     +     |
c            |           |           |           |
c            ------------------|------------------
c            |           |           |           |
c            |     +     -     +     -     +     |
c            |           |           |           |
c            ------------------|------------------
c            |           |           |           |
c            |     +     |     +     |     +     |
c            |           |           |           |
c            -------------------------------------
c
c
c     If the integration point is in the 1-direction (locdx(1)=+/-1)
c     then the contribution in the integration point must be added to
c     the matrix elements at the left and right multiplied by 2,
c     in case of the 2-direction the matrix above and below must be used.
c
c     In case of upwinding the sign of the velocity defines the
c     distribution. If in a 1-direction integration point the flux component
c     is positive, the matrix element to the left is used and so on
c
c
c     The hybrid scheme is a combination of both depending on the Peclet
c     number
c     If |Peclet| <= 1, the central scheme is used, incase of |Peclet| > 1,
c     we use:
c
c        s = 1 - 1/|Peclet|
c        multiplication factor = 0.5 (1-s sign(flux) )
c
c     Higher order upwind schemes are constructed by defect correction,
c     using the upwind scheme as iteration matrix
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Shift from integration cell to tsum; i.e. to cell
c         containing the integration point
c         (Rule: for each grid cell left and lower side are included,
c          top and right hand side are in the next cell)

      shifti = dcell(1)
      shiftj = dcell(2)

c     --- Compute the two connections independent of the velocity direction

      dx(1) = 0
      dx(2) = 0
      dx(alphad) = -1
      ofstnr(1) = mct(shifti,shiftj,0)
      ofstnr(2) = mct(shifti+dx(1),shiftj+dx(2),0)

c     --- Make a choice between central, upwind and hybrid scheme

      if ( ichoisdis.eq.0 ) then

c     --- central scheme

         do jp = iarea(1,2),iarea(2,2)
            do ip = iarea(1,1),iarea(2,1)
               matrix(ip,jp,ofstnr(1)) = matrix(ip,jp,ofstnr(1)) +
     +                      0.5d0 * tsum(ip+shifti,jp+shiftj)
               matrix(ip,jp,ofstnr(2)) = matrix(ip,jp,ofstnr(2)) +
     +                      0.5d0 * tsum(ip+shifti,jp+shiftj)
            end do
         end do

      else if ( ichoisdis.eq.2 .and. idiffus.eq.0 ) then

c     --- hybrid scheme

         if ( alphad.eq.1 ) then
            iptensor1 = 1
         else
            iptensor1 = 3
         end if
         do jp = iarea(1,2),iarea(2,2)
            do ip = iarea(1,1),iarea(2,1)

c           --- Compute the flux, mesh-Peclet number and switching
c               function at the integration point

               flux = veloc(ip+shifti,jp+shiftj)
               pe = ( 1d-8 + cstar(ip+shifti,jp+shiftj)*flux )/
     +              ( 1d-8 + 2d0 * kdif(ip+shifti,jp+shiftj,iptensor1 )*
     +              jac(ip+shifti,jp+shiftj) )
               swfunc = 1d0 - min(1d0,1d0/abs(pe))
               matrix(ip,jp,ofstnr(1)) = matrix(ip,jp,ofstnr(1)) +
     +                      0.5d0 * (1d0 - swfunc * sign(1d0,flux) ) *
     +                      tsum(ip+shifti,jp+shiftj)
               matrix(ip,jp,ofstnr(2)) = matrix(ip,jp,ofstnr(2)) +
     +                      0.5d0 * (1d0 + swfunc * sign(1d0,flux) ) *
     +                      tsum(ip+shifti,jp+shiftj)
            end do
         end do

      else

c     --- upwind scheme

         do jp = iarea(1,2),iarea(2,2)
            do ip = iarea(1,1),iarea(2,1)
               flux = veloc(ip+shifti,jp+shiftj)
               if ( flux.gt.0d0 ) then
                  matrix(ip,jp,ofstnr(2)) = matrix(ip,jp,ofstnr(2)) +
     +                       tsum(ip+shifti,jp+shiftj)
               else
                  matrix(ip,jp,ofstnr(1)) = matrix(ip,jp,ofstnr(1)) +
     +                       tsum(ip+shifti,jp+shiftj)
               end if
            end do
         end do

      end if

      end
