      subroutine ismomcv2 ( ni, nj, nvirtual, densrho, chsym, invjac,
     +                      veloc, vsum, alphad, locdx, iindsc, iparea,
     +                      adir, cva )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.1    date 18-05-1998 Charles Bilinear discretization
c        version 2.0    date 12-06-1996 Wesbeek discretization has been added
c        version 1.0    date 15-03-1995 Subroutine extracted from ismmc2
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     determines the coefficients of the velocity components in
c     a given integration point within a flux cell for the convection term in R2
c **********************************************************************
c
c                       KEYWORDS
c
c    convection
c    matrix
c    right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, alphad, locdx(2), iparea(2,2),
     +        iindsc(*)
      double precision densrho(1-nvirtual:ni+1+nvirtual,
     +                         1-nvirtual:nj+1+nvirtual),
     +                 veloc(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual)
      double precision vsum(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 chsym(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2,3),
     +                 adir(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2)

c     adir      i    Contravariant base vector in integration cell center
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     chsym     i    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     densrho   i    Array containing the density rho. The size and structure is
c                    the same as for the scalar unknowns
c     iindsc    i    reserved
c     invjac    i    1/sqrt(g) in pnttyp
c     iparea    i    Area of integration points where a contribution to
c                    the matrix is to be calculated
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c     vsum     i/o   Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, i, betad, mlin, istwo(2,2), gammad, idiscret,
     +        sign, ish, jsh
      double precision  facs, u(2)

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     facs      Multiplication factor
c     gammad    Direction index
c     i         General loop variable
c     idiscret  Type of discretization
c     ip        Local counting variable i-direction
c     ish       Shift in i-direction
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     jp        Local counting variable j-direction
c     jsh       Shift in j-direction
c     mlin      Type of linearization (1=newton, 2=picard)
c     sign      sign for multiplication (1 or -1)
c     u         Cartesian velocities
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c       Relevant formula
c
c
c    The convective terms in case of the classical discretization are given by:
c
c
c    rho      a  1 |i+1/2,j     rho      a  2 |i,j+1/2
c   ----- V  V  |         + -----    V  V  |          +
c   sqrt(g)        |i-1/2,j    sqrt(g)        |i,j-1/2
c
c
c    rho    |  a  |   c  b |
c   ----- |     |  V  V  |
c   sqrt(g) | b c |        |i,j
c
c
c   There are two linearization methods available Newton and Picard.
c
c   Application of Newton linearization:
c
c   a  b    _a  b      a _b     _a _b
c  V  V  =  V  V   +  V  V   -  V  V
c                                                 1
c   results in the following contribution of the V  component to the matrix:
c
c
c    rho      1 |i+1/2,j     rho      2 |i,j+1/2      rho      a |i+1/2,j
c   ----- V  |         + -----    V  |          + -----    V  |         +
c   sqrt(g)     |i-1/2,j    sqrt(g)     |i,j-1/2     sqrt(g)     |i-1/2,j
c
c
c  2 rho    |  a  |   b |
c   ----- |     |  V  |
c   sqrt(g) | 1 b |     |i,j
c
c                                              2
c  and of course a similar expression for the V  component
c
c  Application of the Picard linearization (in the b-th momentum equation)
c
c   a  b     _a  b
c  V  V  =   V  V
c                                               1
c  results in the following contribution of the V  component to the matrix:
c
c
c    rho      1 |i+1/2,j     rho      2 |i,j+1/2      rho    |  a  |   b |
c   ----- V  |         + -----    V  |         +  -----   |     |  V  |
c   sqrt(g)     |i-1/2,j    sqrt(g)     |i,j-1/2     sqrt(g) | 1 b |     |i,j
c
c                                                2
c    and of course a similar expression for the V  component
c
c  In case of Picard the linearization is as follows:
c
c  In the V^1 equation all terms V^2 are taken at the old level,
c  in the V^2 equation all terms V^1 are taken at the old level.
c
c
c   In case of the Wesbeek discretization we have the following expressions
c   for the convective terms (alpha cell):
c
c     (alpha)           1  |i+1/2,j          2  |i,j+1/2
c    a  (i,j)  . ( rho V u |         +  rho V u |         )
c                          |i-1/2,j             |i,j-1/2
c
c                   1           2
c   with u = (a    V   +  a    V ) / sqrt(g)
c              (1)         (2)
c
c
c   In the Picard linearization we use V at the old level
c   In the Newton linearization we get for the matrix contributions:
c
c     (alpha)           1_ |i+1/2,j          2_ |i,j+1/2
c    a  (i,j)  . ( rho V u |         +  rho V u |          +
c                          |i-1/2,j             |i,j-1/2
c
c
c                      _1  |i+1/2,j         _2  |i,j+1/2
c                  rho V u |         +  rho V u |          )
c                          |i-1/2,j             |i,j-1/2
c
c
c   So the contributions to the matrix become:
c
c                                                  (alpha)      _1
c   Picard: locdx = (+/-1, 0):   V1 term: +/-(rho a (0,0)  . a  V ) / sqrt(g)
c                                                             (1)
c
c                                                  (alpha)      _1
c                                V2 term: +/-(rho a (0,0)  . a  V ) / sqrt(g)
c                                                             (2)
c
c                                                  (alpha)      _2
c           locdx = (0, +/-1):   V1 term: +/-(rho a (0,0)  . a  V ) / sqrt(g)
c                                                             (1)
c
c                                                  (alpha)      _2
c                                V2 term: +/-(rho a (0,0)  . a  V ) / sqrt(g)
c                                                             (2)
c
c   Newton = Picard + extra
c
c                                                   (alpha)
c   Extra:   locdx = (+/-1, 0):   V1 term: +/- rho a (0,0)  . u
c                                                   (alpha)
c            locdx = (0, +/-1):   V2 term: +/- rho a (0,0)  . u
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2,
     +             2, 3 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      idiscret = iindsc(2)
      mlin = iindsc(8)

      if ( idiscret.eq.1 ) then

c     --- idiscret=1, Classical discretization

         if ( mlin.eq.1 ) then

c        --- mlin=1, Newton lin. : multiplier is 2d0.

            facs = 2d0

         else

c        --- mlin=2, Picard lin. : multiplier is 1d0.

            facs = 1d0

         end if

         if ( abs(locdx(1))+abs(locdx(2) ).eq.1 ) then

c        --- centres of integration cell sides
c            betad in (convection term), betad

            if ( locdx(1).ne.0 ) then
               betad = 1
               sign = locdx(1)
            else
               betad = 2
               sign = locdx(2)
            end if

            if ( alphad.eq.betad ) then

c           --- alphad = betad, hence (1,1) or (2,2) element in block matrix

               do 110 jp = iparea(1,2), iparea(2,2)
                  do 100 ip = iparea(1,1), iparea(2,1)
                     vsum(ip,jp,betad) = vsum(ip,jp,betad)+sign*facs*
     +                                   densrho(ip,jp)*invjac(ip,jp)*
     +                                   veloc(ip,jp,alphad)
100               continue
110            continue

            else

c           --- alphad # betad, hence (1,2) and (2,1) element in block matrix

               do 130 jp = iparea(1,2), iparea(2,2)
                  do 120 ip = iparea(1,1), iparea(2,1)
                     vsum(ip,jp,alphad) = vsum(ip,jp,alphad)+sign*
     +                                    densrho(ip,jp)*invjac(ip,jp)*
     +                                    veloc(ip,jp,betad)
120               continue
130            continue

               if ( mlin.eq.1 ) then

c              --- mlin=1, Newton linearisation. No contribution for
c                             Picard.

                  do 150 jp = iparea(1,2), iparea(2,2)
                     do 140 ip = iparea(1,1), iparea(2,1)
                        vsum(ip,jp,betad) = vsum(ip,jp,betad)+sign*
     +                                      densrho(ip,jp)*
     +                                      invjac(ip,jp)*
     +                                      veloc(ip,jp,alphad)
140                  continue
150               continue

               end if

            end if

         else if ( locdx(1).eq.0 .and. locdx(2).eq.0 ) then

c        --- Take care of christoffel symbol term
c            from (convection term),betad derivative in cell centre

            do 180 i = 1, 2
               do 170 jp = iparea(1,2), iparea(2,2)
                  do 160 ip = iparea(1,1), iparea(2,1)
                     vsum(ip,jp,i) = invjac(ip,jp)*densrho(ip,jp)*facs*
     +                              (chsym(ip,jp,alphad,istwo(i,1))*
     +                               veloc(ip,jp,1)+
     +                               chsym(ip,jp,alphad,istwo(i,2))*
     +                               veloc(ip,jp,2))
160               continue
170            continue
180         continue

         end if

c        --- else if ( locdx(1).ne.0 .or. locdx(2).ne.0 ) then

      else if ( (locdx(1).ne.0 .or. locdx(2).ne.0) .and.
     +          (idiscret.eq.2 .or. idiscret.eq.4) ) then

c     --- idiscret=2, Wesbeek discretization
c         The centroid has no contribution at all

         if ( alphad.eq.1 ) then
            ish = 0
            jsh =0
            if ( locdx(1).ne.0 ) then
               betad = 1
               sign = locdx(1)
                if ( sign.lt.0 ) ish = -sign
            else
               betad = 2
               sign = locdx(2)
               if ( sign.gt.0 ) jsh = -sign
            end if
         else
            ish = 0
            jsh =0
            if ( locdx(1).ne.0 ) then
               betad = 1
               sign = locdx(1)
                if ( sign.gt.0 ) ish = -sign
            else
               betad = 2
               sign = locdx(2)
               if ( sign.lt.0 ) jsh = -sign
            end if
         end if

         if ( mlin.eq.1 ) then

c        --- Newton linearization

            gammad = 3-betad
            do jp = iparea(1,2), iparea(2,2)
               do ip = iparea(1,1), iparea(2,1)
                  facs = sign*densrho(ip,jp)*invjac(ip,jp)
                  u(1) = cva(ip,jp,1,1)*veloc(ip,jp,1)+
     +                   cva(ip,jp,1,2)*veloc(ip,jp,2)
                  u(2) = cva(ip,jp,2,1)*veloc(ip,jp,1)+
     +                   cva(ip,jp,2,2)*veloc(ip,jp,2)
                  vsum(ip,jp,betad) = facs*
     +                 (adir(ip+ish,jp+jsh,1)*(u(1)+
     +                     cva(ip,jp,1,betad)*veloc(ip,jp,betad))+
     +                  adir(ip+ish,jp+jsh,2)*(u(2)+
     +                     cva(ip,jp,2,betad)*veloc(ip,jp,betad)))
                  vsum(ip,jp,gammad) = facs*veloc(ip,jp,betad)*
     +                 (adir(ip+ish,jp+jsh,1)*cva(ip,jp,1,gammad)+
     +                  adir(ip+ish,jp+jsh,2)*cva(ip,jp,2,gammad))
               end do
            end do

         else

c        --- Picard linearization

            gammad = 3-betad
            do jp = iparea(1,2), iparea(2,2)
               do ip = iparea(1,1), iparea(2,1)
                  facs = sign*densrho(ip,jp)*invjac(ip,jp)
                  vsum(ip,jp,betad) = facs*veloc(ip,jp,betad)*
     +                 (adir(ip+ish,jp+jsh,1)*cva(ip,jp,1,betad)+
     +                  adir(ip+ish,jp+jsh,2)*cva(ip,jp,2,betad))
                  vsum(ip,jp,gammad) = facs*veloc(ip,jp,betad)*
     +                 (adir(ip+ish,jp+jsh,1)*cva(ip,jp,1,gammad)+
     +                  adir(ip+ish,jp+jsh,2)*cva(ip,jp,2,gammad))
               end do
            end do

         end if

      end if

      end
