      subroutine ismomds2 ( intn, alphad, nintder, cv, didj, nconct )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.0    date 03-10-1996 Guus: extra parameter nconct
c        version 1.0    date 11-10-1995
c
c   copyright (c) 1995-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Make a list in R2 of the number of points for the differentiation and the
c    corresponding connections for each direction in one integration point
c **********************************************************************
c
c                       KEYWORDS
c
c     distribution
c     discretization
c     matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer alphad, nintder(2,2), intn, didj(2,2,4,2), nconct
      double precision cv(2,2,4)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     cv        o    reserved
c     didj      o    Array containing the distances of the finite difference
c                    points to the cell centra in global co-ordinates
c                    didj(alpha,beta,i,j) denotes the distance of the i-th
c                    finite difference point, component j and with respect to
c                    the d V-alpha/ d ksi beta derivative
c     intn      i    Integration point number
c     nconct    i    Number of connections in a row of the matrix
c     nintder   o    Number of points in the distribution of the finite
c                    difference
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, betad, ksign

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     i         General loop variable
c     j         General loop variable
c     ksign     Help parameter to define 1 or -1
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c      Consider a V1 cell with 5 integration points
c
c      The coefficient of the partial derivative of a velocity coefficient
c      in the equation at the integration point is given.
c      In order to compute the partial deivative it is necessary
c      to compute a linear combination of neighbouring points.
c      For example d V2 / dksi1 in a V1 point (pnttyp=3) is created by
c      the linear combination:
c      (V2(i+1,j+1)-V2(i-1,j+1)+V2(i+1,j-1)-V2(i-1,j+1))/2
c
c          *----------------*-----------------*
c          |                |                 |
c          |                |                 |
c          |                |                 |
c          *----------------*-----------------*
c          |        |       |        |        |
c          |        -       -i,j     -        |
c          |        |       |        |        |
c          *----------------*-----------------*
c
c
c      This subroutine gives the number of distribution points as well
c      as the positions in the row of the matrix where the coefficients
c      must be put as function of the direction alphad, and furthermore
c      the coefficients with respect to the linear cobinations
c
c      The positions in the row of the matrix will be called connections
c      They are computed using the array mct.
c      Array mct depends on alphad, the local direction, the point type
c      and also of the position of the point relative to the cell centre.
c      With respect to the V alphad coefficient the position is equal to
c      the local co-ordinate divided by two.
c      With respect to the V betad coefficient the position is equal to
c      the local co-ordinate
c
c      Differentiation formulae:
c
c      V1-cell:  dU^alpha/dksi^beta
c
c      locdx      alpha,beta    n   fact             (di,dj)
c
c      (0,0)       1   1        2  1/2(1,-1)       (1,0) (-1,0)
c                  1   2        2  1/2(1,-1)       (0,1) (0,-1)
c                  2   1        4  1/2(1,-1,1,-1)  (1,1) (-1,1) (1,-1) (-1,-1)
c                  2   2        4  1/2(1,-1,1,-1)  (1,1) (1,-1) (-1,1) (-1,-1)
c
c     (-1,0)       1   1        2     (-1,1)       (-1,0) (0,0)
c                  1   2        4  1/4(1,-1,1,-1)  (-1,1)(-1,-1) (0,1) (0,-1)
c                  2   1        0
c                  2   2        2     (1,-1)       (-1,1) (-1,-1)
c
c      (1,0)       1   1        2     (1,-1)       (1,0)  (0,0)
c                  1   2        4  1/4(1,-1,1,-1)  (1,1) (1,-1) (0,1) (0,-1)
c                  2   1        0
c                  2   2        2     (1,-1)       ( 1,1) ( 1,-1)
c
c     (0,-1)       1   1        4  1/4(1,-1,1,-1)  (1,-1)(-1,-1) (1,0) (-1,0)
c                  1   2        2     (-1,1)       (0,-1) (0,0)
c                  2   1        2     (1,-1)       (1,-1) (-1,-1)
c                  2   2        4  1/4(1,-1,1,-1)  (-1,-1)(-1,-3) (1,1) (-1,-3)
c                  2   2        0  (if 13-point stencil)
c
c     (0,1)        1   1        4  1/4(1,-1,1,-1)  (-1,1)(1,1) (-1,0) (1,0)
c                  1   2        2     (1,-1)       (0,1) (0,0)
c                  2   1        2     (1,-1)       (1,1) (-1,1)
c                  2   2        4  1/4(1,-1,1,-1)  (-1,3)(-1,-1) (1,3) (1,-1)
c                  2   2        0  (if 13-point stencil)
c
c      V2-cell:  dU^alpha/dksi^beta
c
c      locdx      alpha,beta    n   fact             (di,dj)
c
c      (0,0)       2   2        2  1/2(1,-1)       (0,1) (0,-1)
c                  2   1        2  1/2(1,-1)       (1,0) (-1,0)
c                  1   2        4  1/2(1,-1,1,-1)  (1,1) (1,-1) (-1,1) (-1,-1)
c                  1   1        4  1/2(1,-1,1,-1)  (1,1) (-1,1) (1,-1) (-1,-1)
c
c     (0,-1)       2   2        2     (-1,1)       (0,-1) (0,0)
c                  2   1        4  1/4(1,-1,1,-1)  (1,0) (-1,0) (1,-1)(-1,-1)
c                  1   2        0
c                  1   1        2     (1,-1)       (1,-1) (-1,-1)
c
c      (0,1)       2   2        2     (1,-1)       (0,1)  (0,0)
c                  2   1        4  1/4(1,-1,1,-1)  (1,1) (-1,1) (1,0) (-1,0)
c                  1   2        0
c                  1   1        2     (1,-1)       ( 1,1) ( -1,1)
c
c     (-1,0)       2   2        4  1/4(1,-1,1,-1)  (-1,1)(-1,-1) (0,1) (0,-1)
c                  2   1        2     (-1,1)       (-1,0) (0,0)
c                  1   2        2     (1,-1)       (-1,1) (-1,-1)
c                  1   1        4  1/4(1,-1,1,-1)  (-1,-1)(-3,-1) (1,1) (-3,1)
c                  1   1        0  (if 13-point stencil)
c
c     (1,0)        2   2        4  1/4(1,-1,1,-1)  (1,1)(1,-1) (0,1) (0,-1)
c                  2   1        2     (1,-1)       (1,0) (0,0)
c                  1   2        2     (1,-1)       (1,1) (1,-1)
c                  1   1        4  1/4(1,-1,1,-1)  (3,-1)(-1,-1) (3,1) (-1,1)
c                  1   1        0  (if 13-point stencil)
c
c    The array didj contains the shifts di,dj translated to global co-ordinates
c    The tranlation from local co-ordinates to global is as follows:
c
c    V1-cell:  V1 didj_global = didj_local
c              V2   ( 1, 1)_loc   := ( 0, 1)_glob
c                   ( 1,-1)_loc   := ( 0, 0)_glob
c                   (-1, 1)_loc   := (-1, 1)_glob
c                   (-1,-1)_loc   := (-1, 0)_glob
c                   ( 1, 3)_loc   := ( 0, 2)_glob
c                   ( 1,-3)_loc   := ( 0,-1)_glob
c                   (-1, 3)_loc   := (-1, 2)_glob
c                   (-1,-3)_loc   := (-1,-1)_glob
c    V2-cell:  V2 didj_global = didj_local
c              V1   ( 1, 1)_loc   := ( 1, 0)_glob
c                   ( 1,-1)_loc   := ( 1,-1)_glob
c                   (-1, 1)_loc   := ( 0, 0)_glob
c                   (-1,-1)_loc   := ( 0,-1)_glob
c                   ( 3, 1)_loc   := ( 2, 0)_glob
c                   ( 3,-1)_loc   := ( 2,-1)_glob
c                   (-3, 1)_loc   := (-1, 0)_glob
c                   (-3,-1)_loc   := (-1,-1)_glob
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      betad  = 3-alphad
      go to ( 100, 200, 300, 400, 500 ), intn

c     --- Integration cell centre          integration point 1:(0,0)

100      nintder(alphad,alphad) = 2
         nintder(alphad,betad)  = 2
         nintder(betad,alphad)  = 4
         nintder(betad,betad)   = 4
         do i = 1, 2
            do j = 1, 2
               cv(i,j,1) = 0.5d0
               cv(i,j,2) = -0.5d0
               cv(i,j,3) = 0.5d0
               cv(i,j,4) = -0.5d0
            end do
         end do
         ksign = -1
         do i = 1, 2
            ksign = -ksign
            didj(alphad,alphad,i,alphad) = ksign
            didj(alphad,alphad,i,betad) = 0
            didj(alphad,betad,i,alphad) = 0
            didj(alphad,betad,i,betad) = ksign
         end do
         didj(betad,alphad,1,alphad) = 0
         didj(betad,alphad,1,betad) = 1
         didj(betad,alphad,2,alphad) = -1
         didj(betad,alphad,2,betad) = 1
         didj(betad,alphad,3,alphad) = 0
         didj(betad,alphad,3,betad) = 0
         didj(betad,alphad,4,alphad) = -1
         didj(betad,alphad,4,betad) = 0
         didj(betad,betad,1,alphad) = 0
         didj(betad,betad,1,betad) = 1
         didj(betad,betad,2,alphad) = 0
         didj(betad,betad,2,betad) = 0
         didj(betad,betad,3,alphad) = -1
         didj(betad,betad,3,betad) = 1
         didj(betad,betad,4,alphad) = -1
         didj(betad,betad,4,betad) = 0
         go to 1000

c        --- integration point 2: V1:(-1,0),   V2:(0,-1)

200      nintder(alphad,alphad) = 2
         nintder(alphad,betad)  = 4
         nintder(betad,alphad)  = 0
         nintder(betad,betad)   = 2
         cv(alphad,alphad,1) = -1d0
         cv(alphad,alphad,2) =  1d0
         cv(alphad,betad,1)  = 0.25d0
         cv(alphad,betad,2)  =-0.25d0
         cv(alphad,betad,3)  = 0.25d0
         cv(alphad,betad,4)  =-0.25d0
         cv(betad,betad,1)  = 1d0
         cv(betad,betad,2)  =-1d0
         didj(alphad,alphad,1,alphad) = -1
         didj(alphad,alphad,1,betad) = 0
         didj(alphad,alphad,2,alphad) = 0
         didj(alphad,alphad,2,betad) = 0
         ksign = -1
         do i = 1, 2
            ksign = -ksign
            didj(alphad,betad,i,alphad) = -1
            didj(alphad,betad,i,betad) = ksign
            didj(alphad,betad,i+2,alphad) = 0
            didj(alphad,betad,i+2,betad) = ksign
         end do
         didj(betad,betad,1,alphad) = -1
         didj(betad,betad,1,betad) = 1
         didj(betad,betad,2,alphad) = -1
         didj(betad,betad,2,betad) = 0
         go to 1000

c        --- integration point 3: V1:(1,0),   V2:(0,1)

300      nintder(alphad,alphad) = 2
         nintder(alphad,betad)  = 4
         nintder(betad,alphad)  = 0
         nintder(betad,betad)   = 2
         cv(alphad,alphad,1) =  1d0
         cv(alphad,alphad,2) = -1d0
         cv(alphad,betad,1)  = 0.25d0
         cv(alphad,betad,2)  =-0.25d0
         cv(alphad,betad,3)  = 0.25d0
         cv(alphad,betad,4)  =-0.25d0
         cv(betad,betad,1)  = 1d0
         cv(betad,betad,2)  =-1d0
         didj(alphad,alphad,1,alphad) = 1
         didj(alphad,alphad,1,betad) = 0
         didj(alphad,alphad,2,alphad) = 0
         didj(alphad,alphad,2,betad) = 0
         ksign = -1
         do i = 1, 2
            ksign = -ksign
            didj(alphad,betad,i,alphad) = 1
            didj(alphad,betad,i,betad) = ksign
            didj(alphad,betad,i+2,alphad) = 0
            didj(alphad,betad,i+2,betad) = ksign
         end do
         didj(betad,betad,1,alphad) = 0
         didj(betad,betad,1,betad) = 1
         didj(betad,betad,2,alphad) = 0
         didj(betad,betad,2,betad) = 0
         go to 1000

c        --- integration point 4: V1:(0,-1),   V2:(-1,0)

400      nintder(alphad,alphad) = 4
         nintder(alphad,betad)  = 2
         nintder(betad,alphad)  = 2
         nintder(betad,betad)   = 0
         cv(alphad,alphad,1) = 0.25d0
         cv(alphad,alphad,2) =-0.25d0
         cv(alphad,alphad,3) = 0.25d0
         cv(alphad,alphad,4) =-0.25d0
         cv(betad,betad,1) = 0.25d0
         cv(betad,betad,2) =-0.25d0
         cv(betad,betad,3) = 0.25d0
         cv(betad,betad,4) =-0.25d0
         cv(alphad,betad,1)  =-1d0
         cv(alphad,betad,2)  = 1d0
         cv(betad,alphad,1)  = 1d0
         cv(betad,alphad,2)  =-1d0
         didj(alphad,betad,1,alphad) = 0
         didj(alphad,betad,1,betad) = -1
         didj(alphad,betad,2,alphad) = 0
         didj(alphad,betad,2,betad) = 0
         ksign = -1
         do i = 1, 2
            ksign = -ksign
            didj(alphad,alphad,i,alphad) = ksign
            didj(alphad,alphad,i,betad) = -1
            didj(alphad,alphad,i+2,alphad) = ksign
            didj(alphad,alphad,i+2,betad) = 0
         end do
         didj(betad,alphad,1,alphad) = 0
         didj(betad,alphad,1,betad) = 0
         didj(betad,alphad,2,alphad) = -1
         didj(betad,alphad,2,betad) = 0
         if ( nconct.eq.17 ) then

c        --- 17-point stencil only

            didj(betad,betad,1,alphad) = -1
            didj(betad,betad,2,alphad) = -1
            didj(betad,betad,3,alphad) = 0
            didj(betad,betad,4,alphad) = 0

            didj(betad,betad,1,betad) = 0
            didj(betad,betad,2,betad) = -1
            didj(betad,betad,3,betad) = 1
            didj(betad,betad,4,betad) = -1

         end if
         go to 1000

c        --- integration point 5: V1:(0,1),   V2:(1,0)

500      nintder(alphad,alphad) = 4
         nintder(alphad,betad)  = 2
         nintder(betad,alphad)  = 2
         nintder(betad,betad)   = 0
         cv(alphad,alphad,1) = 0.25d0
         cv(alphad,alphad,2) =-0.25d0
         cv(alphad,alphad,3) = 0.25d0
         cv(alphad,alphad,4) =-0.25d0
         cv(betad,betad,1) = 0.25d0
         cv(betad,betad,2) =-0.25d0
         cv(betad,betad,3) = 0.25d0
         cv(betad,betad,4) =-0.25d0
         cv(alphad,betad,1)  = 1d0
         cv(alphad,betad,2)  =-1d0
         cv(betad,alphad,1)  = 1d0
         cv(betad,alphad,2)  =-1d0
         didj(alphad,betad,1,alphad) = 0
         didj(alphad,betad,1,betad) = 1
         didj(alphad,betad,2,alphad) = 0
         didj(alphad,betad,2,betad) = 0
         ksign = -1
         do i = 1, 2
            ksign = -ksign
            didj(alphad,alphad,i,alphad) = ksign
            didj(alphad,alphad,i,betad) = 1
            didj(alphad,alphad,i+2,alphad) = ksign
            didj(alphad,alphad,i+2,betad) = 0
         end do
         didj(betad,alphad,1,alphad) = 0
         didj(betad,alphad,1,betad) = 1
         didj(betad,alphad,2,alphad) = -1
         didj(betad,alphad,2,betad) = 1
         if ( nconct.eq.17 ) then

c        --- 17-point stencil only

            didj(betad,betad,1,alphad) = -1
            didj(betad,betad,2,alphad) = -1
            didj(betad,betad,3,alphad) = 0
            didj(betad,betad,4,alphad) = 0

            didj(betad,betad,1,betad) = 2
            didj(betad,betad,2,betad) = 0
            didj(betad,betad,3,betad) = 2
            didj(betad,betad,4,betad) = 0

         end if

1000  continue

      end
