      subroutine isupwvel2 ( ni, nj, nvirtual, veloc, vectorupw,
     +                       alphad, intpnr )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 30-03-1999 Do not set vectorupw equal to zero for
c                                       points outside the domain because of
c                                       multiblock
c        version 1.0    date 17-03-1998
c
c   copyright (c) 1998-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute the upwind factors (1+sign(V^alpha)) and (1-sign(V^alpha))
c    for the upwinding in the integration points
c **********************************************************************
c
c                       KEYWORDS
c
c     discretization
c     matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, alphad, intpnr
      double precision veloc(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2),
     +                 vectorupw(1-nvirtual:ni+1+nvirtual,
     +                           1-nvirtual:nj+1+nvirtual,2)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     intpnr    i    Integration point under consideration
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     vectorupw o    Array containing the multiplication factors for upwind
c                    vector contains two positions for these factors
c                    The multiplication factor is a vector containing either:
c                    (1,0), (0,1) or (0.5,0.5)
c                    (1,0) means take point left or below integration point
c                    (0,1) means take point right or above integration point
c                    (0.5,0.5) means take mean value (i.e. central differences)
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j
      double precision sigmul, meanvel

c     i         General loop variable
c     j         General loop variable
c     meanvel   Mean value
c     sigmul    Multiplication factor depending on sign
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    The first order upwind method requires the evaluation of the terms
c    (1+sign(V^alpha)) and (1-sign(V^alpha)) in the integration points
c    Here these terms are computed for the integration points
c    intpn = 1, i.e. (+/-1,0) if alpha=1 and (0,+/-1) if alpha=2
c    intpn = 2, i.e. (0,+/-1) if alpha=1 and (+/-1,0) if alpha=2
c
c    If the velocities at both sides have different directions, a central
c    scheme is used
c    Hence if V_left and V_right > 0 use (1,0)
c          if V_left and V_right < 0 use (0,1)
c          if V_left and V_right have different signs use (0.5,0.5)
c
c    The parameter sigmul is used to compute these factors
c    It gets the value 0, 0.5 or -0.5
c    This value is added to or subtracted from 1/2
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      if ( alphad.eq.1 ) then

c     --- alphad = 1

         if ( intpnr.eq.1 ) then

c        --- integration point 1, i.e. (1,0)

            do j = 1, nj
               do i = 0, ni+1
                  meanvel = veloc(i,j,1)+veloc(i+1,j,1)
                  sigmul = sign(0.5d0,meanvel)
                  vectorupw(i,j,1) = 0.5d0+sigmul
                  vectorupw(i,j,2) = 0.5d0-sigmul
               end do
            end do

         else

c        --- integration point 2, i.e. (0,1)

            do i = 1, ni+1
               do j = 1, nj+1
                  meanvel = veloc(i-1,j,2)+veloc(i,j,2)
                  sigmul = sign(0.5d0,meanvel)
                  vectorupw(i,j,1) = 0.5d0+sigmul
                  vectorupw(i,j,2) = 0.5d0-sigmul
               end do
            end do

         end if

      else

c     --- alphad = 2

         if ( intpnr.eq.1 ) then

c        --- integration point 1, i.e. (0,1)

            do i = 1, ni
               do j = 0, nj+1
                  meanvel = veloc(i,j,2)+veloc(i,j+1,2)
                  sigmul = sign(0.5d0,meanvel)
                  vectorupw(i,j,1) = 0.5d0+sigmul
                  vectorupw(i,j,2) = 0.5d0-sigmul
               end do
            end do

         else

c        --- integration point 2, i.e. (1,0)

            do i = 1, ni+1
               do j = 1, nj+1
                  meanvel = veloc(i,j-1,1)+veloc(i,j,1)
                  sigmul = sign(0.5d0,meanvel)
                  vectorupw(i,j,1) = 0.5d0+sigmul
                  vectorupw(i,j,2) = 0.5d0-sigmul
               end do
            end do

         end if

      end if
      end
