      subroutine isvecup2 ( ni, nj, nvirtual, veloc, vecup2, alphad,
     +                      locdx, ibinfor, ibipoint )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 2.0    date 18-01-1999 points outside region changed
c                                       Extra parameters
c        version 1.0    date 14-08-1998
c
c   copyright (c) 1998-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Compute help variable, in integration point, for first order
c   conservative upwind.
c   Wesbeek discretization
c **********************************************************************
c
c                       KEYWORDS
c
c    convection
c    upwind
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, alphad, locdx(2), ibipoint(*),
     +        ibinfor(*)
      double precision veloc(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,2),
     +                  vecup2(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,2)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     vecup2    o    Upwind help vector 2, (1+sign V)/2 and (1-sign V)/2 in
c                    integration point
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer btype, ipibinf, ip, jp

c     btype     Type of boundary condition
c     ip        Local counting variable i-direction
c     ipibinf   Starting address of array infor for present side minus one
c     jp        Local counting variable j-direction
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c       Relevant formula
c
c
c   In case of the Wesbeek discretization we have the following expressions
c   for the convective terms (alpha cell):
c
c     (alpha)           1  |i+1/2,j          2  |i,j+1/2
c    a  (i,j)  . ( rho V u |         +  rho V u |         )
c                          |i-1/2,j             |i,j-1/2
c
c                   1           2
c   with u = (a    V   +  a    V ) / sqrt(g)
c              (1)         (2)
c
c
c   In the Picard linearization we use V at the old level
c
c   For first order upwind the contributions are computed in
c   both upwind points ( both directions ) and  multiplied
c   with a factor equals 1 or 0, depending on the sign of the
c   flux in the integration point. the factor is computed in
c   this routine.
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

c     --- initialize vecup2

      do ip = 1-nvirtual, ni+1+nvirtual
         do jp = 1-nvirtual, nj+1+nvirtual
            vecup2(ip,jp,1) = 0d0
            vecup2(ip,jp,2) = 0d0
         end do
      end do

      if ( locdx(1).ne.0 ) then

         do ip = 1-nvirtual+1, ni+1
            do jp = 1-nvirtual+1, nj+1
               vecup2(ip,jp,1) = (1d0+sign(1d0,veloc(ip,jp,1)) )/2d0
               vecup2(ip,jp,2) = (1d0-sign(1d0,veloc(ip,jp,1)) )/2d0
            end do
         end do

         if ( alphad.eq.1 ) then

c        --- First side 1, points with i = 0, are outside the region
c             Only the points that do not have periodical or multiblock
c             boundary conditions should be treated as special

           ip = 0
           ipibinf = ibipoint(1)+nvirtual-1
           do jp = 1-nvirtual+1, nj+1
              btype = ibinfor(ipibinf+2*jp-1)
              if ( btype.lt.90 ) then
                 vecup2(ip,jp,1) = 0d0
                 vecup2(ip,jp,2) = 1d0
              end if
           end do

c          --- Next side 2, points with i = ni+1, are outside the region

           ip = ni+1
           ipibinf = ibipoint(2)+nvirtual-1
           do jp = 1-nvirtual+1, nj+1
              btype = ibinfor(ipibinf+2*jp-1)
              if ( btype.lt.90 ) then
                 vecup2(ip,jp,1) = 1d0
                 vecup2(ip,jp,2) = 0d0
              end if
           end do
        end if

      else if ( locdx(2).ne.0 ) then

         do ip = 1-nvirtual+1, ni+1
            do jp = 1-nvirtual+1, nj+1
               vecup2(ip,jp,1) = (1d0+sign(1d0,veloc(ip,jp,2)) )/2d0
               vecup2(ip,jp,2) = (1d0-sign(1d0,veloc(ip,jp,2)) )/2d0
            end do
         end do

         if ( alphad.eq.2) then

c        --- First side 3, points with j = 0, are outside the region
c            Only the points that do not have periodical or multiblock
c            boundary conditions should be treated as special

            jp = 0
            ipibinf = ibipoint(3)+nvirtual-1
            do ip = 1-nvirtual+1, ni+1
               btype = ibinfor(ipibinf+2*ip-1)
               if ( btype.lt.90 ) then
                  vecup2(ip,jp,1) = 0d0
                  vecup2(ip,jp,2) = 1d0
               end if
            end do
            jp = nj+1
            ipibinf = ibipoint(4)+nvirtual-1
            do ip = 1-nvirtual+1, ni+1
               btype = ibinfor(ipibinf+2*ip-1)
               if ( btype.lt.90 ) then
                  vecup2(ip,jp,1) = 1d0
                  vecup2(ip,jp,2) = 0d0
               end if
            end do
         end if

      end if

      end
