      subroutine isstre2d ( work, pderiv, veloc, pveloc, solut, eddy,
     +                      cva, pcva, invjac, locinvjac, hightens,
     +                      stress, jcta, jctg, chsym, ni, nj, ndegfd,
     +                      nvirtual, idiscret, pnttyp, rintur )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 2.1    date   18-05-1998 Charles Bilinear discretization
c        version 2.0    date   11-09-1996 Marcel: nonlinear effects
c        version 1.2    date   11-09-1996 Marcel: bug fixed
c        version 1.1    date   10-04-1996 Guus: remove last dimension of solut
c        version 1.0    date   05-03-1996
c
c   copyright (c) 1996-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute the Reynolds stresses, as follows:
c     -----
c     ^a ^b   2  ab             ac  b     bc  a
c     U  U  = - g   k - nu_t ( g   U   + g   U  )
c             3                     ,c        ,c
c     If necessary, add some nonlinear (anisotropic) effects
c     Afterwards, translate to the Cartesian system
c     Otherwise, compute the Reynolds stresses using WESBEEK method
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     turbulence
c     write
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, ndegfd, nvirtual, idiscret, pnttyp
      double precision work(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,1:3),
     +                 hightens(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual),
     +                 pderiv(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1:2,1:2),
     +                 veloc(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:2,1:3),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd)
      double precision cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2,4),
     +                 stress(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,2,2),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,4),
     +                 jcta(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2,2),
     +                 jctg(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,3),
     +                 chsym(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2,3)
      double precision locinvjac(*), pveloc(*), pcva(*), rintur(*)

c     chsym     i    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     eddy      i    reserved
c     hightens  i    Higher order tensor constains quadratic products of
c                    strain and rotation tensors
c     idiscret  i    Type of discretization
c     invjac    i    1/sqrt(g) in pnttyp
c     jcta      i    Jacobian times contravariant basis vectors
c                    in point pnttyp
c                    (stored as vector index encoding given by istwo)
c     jctg      i    Jacobian times symmetric contravariant metric
c                    tensor in point given by pnttyp
c                    (stored as vector index encoding given by istwo)
c     locinvjac i    Array containing the inverse of the Jacobian for all
c                    integration points of one type
c     ndegfd    i    Number of degrees of freedom
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     pcva      i    Pointer to work array containing covariant base vectors
c     pderiv    i    Partial derivative of flux in point pnttyp
c     pnttyp    i    reserved
c     pveloc    i    Pointer to work array for the old velocities (used in
c                    convection term calculation)
c     rintur    i    reserved
c     solut     i    reserved
c     stress    i    A local array containing physical stresses
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c     work      o    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, alphad, alphal, betad, betal, deltal, i, j, idir,
     +        dx(1:2), gammal, vcellc, dummy1, istwo(1:2,1:2),
     +        iarea(2,2)
      double precision kroneck, contrst, u(1:2,1:2,1:2),
     +                 dudx(1:2,1:2)

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     alphal    Loop index
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     betal     Loop index
c     contrst   Contravariant rate of strain tensor
c     deltal    Loop index
c     dudx      Partial derivative of Cartesian velocities in Cartesian
c               directions
c     dummy1    Dummy
c     dx        Point location in cell relative to local
c               coordinates with (0,0) and (2,2) or
c               (0,0,0) and (2,2,2) as opposite cell corners
c     gammal    Loop index
c     i         General loop variable
c     iarea     Area of arrays
c     idir      General loop variable for 1...ndim loop
c     ip        Local counting variable i-direction
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     j         General loop variable
c     jp        Local counting variable j-direction
c     kroneck   Kronecker delta
c     u         Cartesian velocities
c     vcellc    Point type of integration cell centre for a given
c               variable type
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISQUADTN  Compute the quadratic products of strain and rotation
c               tensors in R2
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2,
     +             2, 3 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      iarea(1,1) = 1
      iarea(1,2) = 1
      iarea(2,1) = ni
      iarea(2,2) = nj

      if ( idiscret.eq.1 ) then

c     --- standard discretization

        do alphad = 1, 2
          do betad = 1, 2

c         --- Determine the quadratic products of strain and
c             rotation tensors

            call isquadtn( hightens, alphad, betad, pderiv,
     +                     pveloc, pcva, locinvjac, jctg, chsym,
     +                     ni, nj, nvirtual, idiscret, rintur(22),
     +                     rintur(23), rintur(24), iarea )

            do jp = 1, nj
              do ip = 1, ni

                contrst = 0d0

                do gammal = 1, 2

                  contrst = contrst + invjac(ip,jp,pnttyp) *
     +                      invjac(ip,jp,pnttyp) *
     +                      ( jctg(ip,jp,istwo(alphad,gammal)) *
     +                        pderiv(ip,jp,betad,gammal) +
     +                        jctg(ip,jp,istwo(betad,gammal)) *
     +                        pderiv(ip,jp,alphad,gammal) )

                  do deltal = 1, 2

                    contrst = contrst + invjac(ip,jp,pnttyp) *
     +                  invjac(ip,jp,pnttyp) *
     +                  ( jctg(ip,jp,istwo(alphad,gammal)) *
     +                    ( chsym(ip,jp,betad,istwo(deltal,gammal))*
     +                      veloc(ip,jp,deltal,3) -
     +                      chsym(ip,jp,deltal,istwo(gammal,deltal))*
     +                      veloc(ip,jp,betad,3) ) +
     +                    jctg(ip,jp,istwo(betad,gammal)) *
     +                    ( chsym(ip,jp,alphad,istwo(deltal,gammal))*
     +                      veloc(ip,jp,deltal,3) -
     +                      chsym(ip,jp,deltal,istwo(gammal,deltal))*
     +                      veloc(ip,jp,alphad,3) ) )

                  end do

                end do

                stress(ip,jp,alphad,betad)= 2d0*invjac(ip,jp,pnttyp)*
     +                             jctg(ip,jp,istwo(alphad,betad))*
     +                             solut(ip,jp,ndegfd-1)/3d0 -
     +                             eddy(ip,jp)*contrst + eddy(ip,jp)*
     +                             eddy(ip,jp)*hightens(ip,jp)/
     +                             (rintur(3)*solut(ip,jp,ndegfd-1))

              end do
            end do

          end do
        end do

        do jp = 1, nj
          do ip = 1, ni

            do deltal = 1, 2
              do gammal = 1, 2

                dummy1 = gammal + deltal - 1

                work(ip,jp,dummy1) = 0d0

                do alphad = 1, 2
                  do betad = 1, 2
                    work(ip,jp,dummy1) = work(ip,jp,dummy1) +
     +                                  cva(ip,jp,deltal,alphad,pnttyp)*
     +                                  cva(ip,jp,gammal,betad,pnttyp)*
     +                                  stress(ip,jp,alphad,betad)

                  end do
                end do

              end do
            end do

          end do
        end do

      else if ( idiscret.eq.2 .or. idiscret.eq.4 ) then

c     --- WesBeek discretization

        do jp = 1, nj
          do ip = 1, ni

            do idir = 1, 2

c           --- Set vcellc
c               (translate from variable type to corresponding
c                integration cell centre point type)

              vcellc = 4 - 2**(idir-1)

              do i = 1, 2

                dx(1) = 0
                dx(2) = 0
                dx(idir) = i - 1

                do betal = 1, 2

                  u(idir,i,betal) = 0d0

                  do j = 1, 2
                    u(idir,i,betal) = u(idir,i,betal) +
     +                                cva(ip+dx(1),jp+dx(2),betal,j,
     +                                    vcellc) *
     +                                veloc(ip+dx(1),jp+dx(2),j,idir)
                  end do

                  u(idir,i,betal) = u(idir,i,betal)*
     +                              invjac(ip+dx(1),jp+dx(2),vcellc)

                end do
              end do
            end do

            do alphal = 1, 2
              do betal = 1, 2

                dudx(alphal,betal) = 0d0

                do j = 1, 2

                  dudx(alphal,betal) = dudx(alphal,betal) +
     +                                 jcta(ip,jp,betal,j) *
     +                                 invjac(ip,jp,pnttyp) *
     +                                 ( u(j,2,alphal) - u(j,1,alphal) )

                end do

              end do
            end do

            do alphal = 1, 2
              do betal = 1, 2

                kroneck = 0d0
                if ( alphal.eq.betal ) kroneck = 1d0

                dummy1 = alphal + betal - 1

                work(ip,jp,dummy1)  = 2d0*kroneck*
     +                        solut(ip,jp,ndegfd-1)/3d0 -
     +                        eddy(ip,jp) *
     +                        (dudx(alphal,betal) + dudx(betal,alphal))

              end do
            end do

          end do
        end do

      end if

      end
