      subroutine issolv ( iequa, iinsol, rinsol, amat, rhsd, solut,
     +                    solutv, nsolut, nusol, ndegfd, infmat, work,
     +                    nwork, prec, nprec, iinmtvc )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 2.2    date 08-07-1998 Kees V: nusol, nsolut cleaned
c        version 2.1    date 20-10-1997 Kees K. iinmtvc added
c        version 2.0    date 13-10-1997 Charles extra parameter coupled
c        version 1.9    date 19-09-1997 Kees V. changes for uncoupled
c        version 1.8    date 06-08-1997 Guus: return in case of error
c        version 1.7    date 17-04-1997 Kees V. Multi-grid prec added
c        version 1.6    date 23-01-1997 Kees V. CGS is allowed
c        version 1.5    date 27-06-1996 Kees V. Bi-CGSTAB is allowed
c        version 1.4    date 23-05-1996 Kees V. call of isprechk changed
c        version 1.3    date 07-11-1995 Kees V. better error handling
c        version 1.2    date 03-10-1995 Kees K. solutv added
c        version 1.1    date 15-09-1995 Kees V. adding ILU_Fill
c        version 1.0    date 29-06-1995
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     The subroutine issolv is used to solve an unsymmetric system
c     of equations of the shape A x = f.
c     At present the following solution methods are available:
c     CGS    the conjugate gradient-squared method,
c     CGSTAB the Bi-CGSTAB method,
c     GMRES  the generalized minimal residual method,
c     GMRESR the GMRESR method,
c     LSQR   the CG method applied to the normal equations.
c **********************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     preconditioner
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          iequa, ndegfd, nsolut, nusol, nwork, nprec,
     +                 iinsol(*), infmat(*), iinmtvc(*)
      double precision amat(*), rhsd(*), prec(*), work(nwork),
     +                 rinsol(*), solut(nsolut), solutv(*)

c     amat      i    The coefficients matrix used in the linear system
c     iequa     i    General equation sequence number
c     iinmtvc   i    reserved
c     iinsol    i    reserved
c     infmat    i    Integer information for the matrix
c     ndegfd    i    Number of degrees of freedom
c     nprec     i    Maximum number of diagonals in preconditioning matrix
c     nsolut    i    Number of unknowns for all degrees of freedom in one block
c     nusol     i    Number of unknowns for one degree of freedom in
c                    one block
c     nwork     i    Second dimension of array work
c     prec      i    The preconditioning matrix
c     rhsd      i    Vector containing the right-hand side vector of the
c                    system of equations.
c     rinsol    i    reserved
c     solut     i    reserved
c     solutv    i    Work array for solut with virtual points
c     work      i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer          method, irrind, itmax, istop, iprinlin,
     +                 nusol2, icontr(9), lwork, ndim, ndimso,
     +                 nconct, ipstrt, ni, i, initer(400), precblt,
     +                 saveprec, lworks
      logical          conv, coupled
      double precision eps1, eps2, rhow, dt, reiter(100)

c     conv      Logical variable indicating acquired convergence.
c     coupled   Logical which is true if the momentum equations
c               are coupled
c     dt        Time step
c     eps1      Determine the accuracy of the final approximation. The
c               termination criterion is based upon
c               ||b-Ax || <  eps1 + eps2 * (b-Ax ) .
c                     k                         0
c     eps2      Determine the accuracy of the final approximation. The
c               termination criterion is based upon
c               ||b-Ax || <  eps1 + eps2 * (b-Ax ) .
c                     k                         0
c     i         General loop variable
c     icontr    reserved
c     initer    reserved
c     iprinlin  Control parameter indicating the amount of output.
c     ipstrt    Choice parameter for the startvector
c                 IPSTRT = iinsol(4) ;
c                 IPSTRT = 0 : Startvector = 0
c                 IPSTRT = 2 : Startvector = taken from the previous
c                                            multi-block iteration
c                                            or time step.
c     irrind    Variable indicating possible failures.
c     istop     Indicates which actions should be done in case of termination
c     itmax     The maximum number of iterations to be performed in
c               the linear solver.
c     lwork     Length of array work
c     lworks    Length work array for solver
c     method    Control parameter indicating the iterative method.
c     nconct    Number of connections in a row of the matrix
c     ndim      Dimension of space (2 or 3)
c     ndimso    Integer indicating the dimension of the space in
c               which the problem must be solved (ndimso = 1 or ndim)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nusol2    Number of scalar unknowns for the solvers
c               nusol2 = nusol * ndimso
c     precblt   The preconditioner has been built (=1) or not (=0)
c     reiter    reserved
c     rhow      Specific weight
c     saveprec  Save preconditioner across domain decomposition
c               iterations (=1) or not (=0)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     GMRESR    Computes the solution of an unsymmetric system of linear
c               equations using polynomial preconditioning by GMRES.
c     GMRITZ    Calculates the first itmax Ritz values for the (pre-
c               conditioned) matrix.
c     ISCGS     Computes the solution of an unsymmetric system of linear
c               equations. This is the conjugate gradient-squared method.
c     ISCGSTAB  Computes the solution of an unsymmetric system of linear
c               equations. This is the Bi-CGSTAB method.
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRSUB  Error messages
c     ISEXTR    Compute the initial guess vector for subdomain solution
c     ISGMRES   Computes the solution of an unsymmetric system of linear
c               equations. This is the generalized minimal residual method.
c     ISMKPREC  Builds the preconditioner.
c     ISPRECHK  This subroutine checks if the requested preconditioner is
c               implemented.
c     ISPREVC   Computes the product of the preconditioner, stored in
c                  amat and prec.
c     ISPUTS    Put the computed solution into solut
c     ISSCALMA  Subroutine to scale the matrix
c     ISSTOP    Stop ISNAS
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1002   work space is too small
c    1004   no convergence in linear solver
c    1010   No linear solver has been called.
c **********************************************************************
c
c                       PSEUDO CODE
c    We start with a call to isextr, which initializes the starting
c    vector in work(1:nusol2). If necessary a preconditioner is built
c    by the subroutine ismkprec. Finally if the iterative
c    method has converged the solution is copied from work(1:nusol2)
c    into solut by a call of isputs.
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

chpu      on real underflow ignore
      call iseropen('issolv')

c     --- fill ndim, ndimso, nusol2, nconct

      if ( infmat(4).eq.0 ) then
         ndim = 2
      else
         ndim = 3
      end if

      coupled = iinsol(25).eq.0

      if ( infmat(8).eq.1 .and. coupled ) then
         ndimso = ndim
      else
         ndimso = 1
      end if

      nusol2 = nusol * ndimso
      nconct = infmat(6)

c     --- Initialize starting vector in work(1:nusol2);

      ipstrt = iinsol(4)
      method = iinsol(7)

      if ( method.ne.6 ) then
         call isextr ( iequa, solut, infmat, work, nusol, nsolut,
     +                 ndegfd, ipstrt, ndim, amat, rhsd, nconct,
     +                 nusol2, coupled )
      end if
      if ( nerror.ne.0 ) go to 1000

c     --- Initialization of constants and the arrays initer and reiter.

      do i = 1, 400
         initer(i) = 0
      end do

      do i = 1, 100
         reiter(i) = 0d0
      end do

      initer(1) = iinsol(2)
      initer(2) = iinsol(3)
      initer(3) = 1
      initer(5) = iinsol(5)
      initer(6) = iinsol(11)
      initer(7) = iinsol(12)
      initer(8) = iinsol(13)
      initer(9) = iinsol(7)
      initer(10)= iinsol(6)
      initer(11)= infmat(8)
      initer(12)= infmat(2)
      initer(13)= infmat(3)
      initer(14)= infmat(4)
      initer(15)= ndim
      initer(16)= ndimso
      initer(17)= nprec
      initer(18)= infmat(6)
      initer(19)= 25
      initer(20)= 10
      initer(21)= iinsol(21)
      initer(22)= iinsol(22)
      initer(23)= infmat(1)
      initer(24)= iinsol(23)
      initer(25)= iinsol(24)
      initer(26)= iinsol(9)
      initer(27)= 101
      initer(28)= iinsol(10)
      initer(29)= iinsol(25)

      reiter(1) = rinsol(1)
      reiter(2) = rinsol(2)
      reiter(3) = rinsol(5)
      reiter(7) = rinsol(7)
      reiter(8) = rinsol(8)

      itmax     = initer(10)
      eps1      = rinsol(1)
      eps2      = rinsol(2)
      icontr(1) = initer(1)
      icontr(2) = initer(2)
      icontr(5) = iinsol(8)
      istop     = initer(5)
      iprinlin  = initer(2)
      ni        = infmat(2)
      saveprec  = iinsol(15)
      precblt   = iinsol(16)
      irrind    = 0
      conv      = .false.

c     --- Below lworks is filled to test if sufficient workspace is
c         available for the printing routine. The workspace is the
c         sum of the maximal number of iterations (itmax + 2), the
c         information in array initer (100), and the number of inner
c         iterations in GMRESR.

      lworks    = itmax + 102 + initer(8)

c     --- If ni = 2 then the ILU-preconditioning applied to the
c         pressure equations can give breakdown, because the
c         pressure equation is singular and for ni = 2 ILU is
c         equal to the LU-decomposition. In such a case we make
c         initer(1) = -2.

      if ( infmat(8).eq.3 .and. abs(initer(1)).eq.3
     +     .and. ni.eq.2 ) then
         icontr(1) = -2
         initer(1) = -2
      end if

      if ( saveprec.eq.0 .or. saveprec.eq.1 .and.
     +     precblt.eq.0 ) then

c     --- Check if the requested preconditioner is implemented
c         and the matrix is checked.

         call isprechk ( initer, irrind, amat, nusol2 )
         if ( nerror.ne.0 ) go to 1000
         iinsol(2) = initer(1)
         if ( irrind.eq.1 ) then
            initer(3) = 1
            goto 999
         end if

c        --- Check if there is enough memory

         if ( abs(initer(1)).eq.4 .or.
     +        abs(initer(1)).eq.3 .and. initer(15).eq.3 ) then
            lwork = 3 * nusol2

            if ( (lwork+lworks).gt.nwork ) then
               if ( istop.eq.0 ) then
                  if ( iprinlin.ge.0 ) then
                     call iserrsub ( 1002, 0, 0, 0 )
                  end if
                  call isstop
               else
                  if ( iprinlin.ge.0 ) then
                     call iswarsub ( 1002, 0, 0, 0 )
                  end if
               end if
               initer(3) = 2
            end if
         end if
         if ( nerror.ne.0 ) go to 1000
         call ismkprec ( amat, nusol , prec, initer, reiter,
     +                   work(nusol2+1) )
         if ( nerror.ne.0 ) go to 1000
         if ( saveprec.eq.1 ) then
            precblt    = 1
            iinsol(16) = precblt
         end if
      end if

c     --- If necessary the matrix is scaled

      if ( initer(1).eq.2 ) then
         call isscalma ( amat, nusol, ndimso, nconct, prec(nusol2+1) )
         do i = 1, nusol2
            amat(i) = amat(i)-2
         end do
      end if
      if ( nerror.ne.0 ) go to 1000

      if ( method.eq.1 .and. initer(8) .eq.0 ) then

c     --- Solve using CGS
c         10*n should be less than maxwork. The extra space for the
c         residuals is made in islsds02.

         lwork = 12 * nusol2

         if ( (lwork+lworks).gt.nwork ) then
            if ( istop.eq.0 ) then
               if ( iprinlin.ge.0 ) then
                  call iserrsub ( 1002, 0, 0, 0 )
               end if
               call isstop
            else
               if ( iprinlin.ge.0 ) then
                  call iswarsub ( 1002, 0, 0, 0 )
               end if
            end if
         else

            call iscgs ( nusol2, amat, rhsd, work,
     +         work(nusol2+1), work(2*nusol2+1),
     +         work(3*nusol2+1), work(4*nusol2+1), work(5*nusol2+1),
     +         work(6*nusol2+1), work(7*nusol2+1), solutv,
     +         infmat, prec, work(lwork+1), initer, reiter, iinmtvc )

         end if
      end if
      if ( nerror.ne.0 ) go to 1000

c     --- end solve using CGS

      if ( method.eq.2 .and. initer(8).eq.0 ) then

c     --- Solve using GMRES
c         After initer(6) steps the GMRES procedure is restarted
c         (initer(6)+5)*n should be less than maxwork. The extra
c         space for the residuals is made in islsds02.

         lwork = (initer(6)+5)*nusol2

         if ( (lwork+lworks).gt.nwork ) then
            if ( istop.eq.0 ) then
               if ( iprinlin.ge.0 ) then
                  call iserrsub ( 1002, 0, 0, 0 )
               end if
               call isstop
            else
               if ( iprinlin.ge.0 ) then
                  call iswarsub ( 1002, 0, 0, 0 )
               end if
            end if
         else

            call isgmres ( nusol2, amat, rhsd, work,
     +            work(nusol2+1), solutv, infmat, prec, work(lwork+1),
     +            initer, reiter, iinmtvc )

         end if
      end if
      if ( nerror.ne.0 ) go to 1000

c     --- end solve using GMRES

      if ( method.eq.4 .and. initer(8) .eq.0 ) then

c     --- Solve using Bi-CGSTAB
c         10*n should be less than maxwork. The extra space for the
c         residuals is made in islsds02.

         lwork = 10 * nusol2

         if ( (lwork+lworks).gt.nwork ) then
            if ( istop.eq.0 ) then
               if ( iprinlin.ge.0 ) then
                  call iserrsub ( 1002, 0, 0, 0 )
               end if
               call isstop
            else
               if ( iprinlin.ge.0 ) then
                  call iswarsub ( 1002, 0, 0, 0 )
               end if
            end if
         else

            call iscgstab ( nusol2, amat, rhsd, work,
     +         work(nusol2+1), work(2*nusol2+1),
     +         work(3*nusol2+1), work(4*nusol2+1), work(5*nusol2+1),
     +         work(6*nusol2+1), work(7*nusol2+1), solutv,
     +         infmat, prec, work(lwork+1), initer, reiter, iinmtvc )

         end if
      end if
      if ( nerror.ne.0 ) go to 1000

c     --- end solve using Bi-CGSTAB

      if ( initer(8).gt.0 ) then

c     --- Solve using GMRESR
c         If initer(8) > 0 we use the GMRESR method. At this moment
c         this method is only implemented with GMRES as innerloop.
c         initer(8) is equal to the number of innerloop iterations.
c         For GMRES as innerloop this quantity should be less than 100.
c         After initer(6) outer iterations the GMRESR method is re-
c         started, and truncated after initer(7) vectors. In this case
c         (2*initer(7)+initer(8)+10)*nusol2 should be less than
c         maxwrk.

         icontr(6) = initer(6)
         icontr(7) = initer(7)
         icontr(8) = initer(8)
         icontr(9) = method
         lwork = (2*initer(7)+initer(8)+10)*nusol2

         if ( (lwork+lworks).gt.nwork ) then
            if ( iprinlin.ge.0 ) then
               call iserrsub ( 1002, 0, 0, 0 )
            end if
            initer(3) = 2
         else
            call gmresr ( nusol2, amat, rhsd, work(1),solutv, eps1,
     +           eps2, itmax, work(nusol2+1), icontr, infmat,
     +           prec, nprec, ndimso, nconct, iinmtvc )
         end if
      end if
      if ( nerror.ne.0 ) go to 1000

c     --- end solve using GMRESR

      if ( method.eq.0 .and. initer(8).eq.0 ) then

c     --- Solve using LSQR.
c         No preconditioning is allowed at this moment

        lwork = 5 * nusol2
        if ( (lwork+lworks).lt.lwork ) then
           if ( iprinlin.ge.0 ) then
              call iserrsub ( 1002, 0, 0 , 0 )
           end if
           initer(3) = 2
        else

c       --- call lsqr ( nusol2, matrix, precon, rhside, work(1), eps1, eps2,
c           +         itmax, work(nusol2+1), work(2*nusol2+1), work(3*nusol2+1),
c           +         work(4*nusol2+1), icontr, infmat )

        end if
      end if
      if ( nerror.ne.0 ) go to 1000

c     --- end solve using LSQR.

      if (method.eq.6 .and. initer(8) .eq.0 ) then

c     --- Solve using one multiplication by the preconditioner.

         lwork = nusol2

         if ( (lwork+lworks).gt.nwork ) then
            if ( iprinlin.ge.0 ) then
               call iserrsub ( 1002, 0, 0, 0 )
            end if
            initer(3) = 2
         else
            call isprevc ( nusol2, rhsd, work, amat, prec,
     +                     infmat, initer )
            initer(3) = 0
         end if
      end if
999   if ( nerror.ne.0 ) go to 1000

c     --- end solve using one multiplication by the preconditioner.
c         Check for normal termination.

      irrind = irrind + initer(3)
      if ( initer(3).eq.0 ) conv = .true.

c     --- If necessary the matrix is scaled back

      if ( initer(1).eq.2 ) then
         do i = 1, nusol2
            amat(i) = amat(i)+2
         end do
         call isscalma ( amat, nusol, ndimso, nconct, prec )
      end if
      if ( nerror.ne.0 ) go to 1000

c     --- Copy solution from work(1:nusol2)
c         into it position in the second half of the space solut(*),
c         in case of convergence.

      if ( conv ) then
         call isputs ( iequa, solut, nsolut, work, infmat, coupled )
      else
         if ( istop.eq.0 ) then
            if ( iprinlin.ge.0 ) then
               if ( initer(3).eq.1 ) then
                  call iserrsub ( 1010, 0, 0, 0 )
               end if
               if ( initer(3).eq.3 ) then
                  call iserrsub ( 1004, 0, 0, 0 )
               end if
            end if
            call isstop
         else
            if ( iprinlin.ge.0 ) then
               if ( initer(3).eq.1 ) then
                  call iswarsub ( 1010, 0, 0, 0 )
               end if
               if ( initer(3).eq.3 ) then
                  call iswarsub ( 1004, 0, 0, 0 )
               end if
            end if
            call isputs ( iequa, solut, nsolut, work, infmat, coupled )
         end if
      end if
      if ( nerror.ne.0 ) go to 1000

c     --- Compute Ritz values

      if ( iinsol(14).gt.0 ) then

c     --- after iinsol(14) steps the GMRITZ procedure is stopped.
c         (iinsol(14)+5)*n should be less than maxwork.

         lwork = (iinsol(14)+5)*nusol2

         if ( (lwork+lworks).gt.nwork ) then
            if ( iprinlin.ge.0 ) then
               call iserrsub ( 1002, 0, 0, 0 )
            end if
            initer(3) = 2
         else
            itmax = iinsol(14)
            rhow   = rinsol(8)
            dt    = rinsol(4)
            write(6,1010) rhow
            write(6,1020) dt
1010        format(' rhow  = ',e15.5)
1020        format(' dt   = ',e15.5)
            call gmritz ( nusol2, amat, rhsd, work(nusol2+1),
     +           solutv, itmax, work(2*nusol2+1), icontr, infmat,
     +           prec, nprec, ndimso, nconct, rhow, dt , iinmtvc)
         end if
      end if

c     --- end compute Ritz values

1000  call iserclos('issolv')

chpu      on real underflow abort

      end
