      subroutine polgmr(n,amat,rhsd,usol,solutv,r,eps,imax,work,icontr,
     +                 infmat , prec, nprec,ndim,nconct, iinmtvc)
c ======================================================================
c
c     programmer       Kees Vuik
c     version 1.4      date 21-11-1997 Kees K. iinmtvc added
c     version 1.3      date 31-10-1997 Kees V. changes for uncoupled
c     version 1.2      date 03-10-1995 Kees K. solutv added
c     version 1.1      date 04-04-1995 Guus replace mtvc1 by ismatvec
c     version 1.0      date 07-08-1992
c     developed at     Convex,HP700
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     The subroutine polgmr is used as an innerloop for GMRESR. It
c     uses the GMRES method with preconditioning.
c **********************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     preconditioner
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
c     amat    i   Matrix from the equations to be solved.
c     eps     i   Determine the accuracy of the final approximation.
c     icontr  i   Integer array in which information about the solution
c                 process must be given by the user. Icontr must be
c                 filled as follows:
c             1 i Parameter controlling the preconditioning. Possible
c                 values are:
c                  0: No preconditioning.
c                  1: Diagonal preconditioner.
c                 -1: Diagonal postconditioner.
c                  2: ILUD preconditioner, with Eisenstat implemen-
c                     tation.
c                 -2: ILUD postconditioner.
c                  3: ILU preconditioner.
c                 -3: ILU postconditioner.
c             2 i Control parameter indicating the amount of output
c                 required. Possible values:
c                 <0: No output.
c                  1: Only fatal errors will be printed.
c                  2: Additional information about the iteration is printed.
c                  3: Gives a maximal amount of output concerning the
c                     iteration process.
c             5 i Parameter indicating the file number to which output
c                 should be written.
c     iinmtvc i   reserved
c     imax    i   The maximum number of iterations to be performed.
c     infmat  i   Integer array with information of the matrix structure,`
c                 to be used in matrix-vector multiplication subroutine
c     n       i   The number of rows in the matrix A.
c     nconct  i   Maximal number of connections in one row of the
c                 matrix.
c     ndim    i   Integer indicating the amount of unknowns in every
c                 gridpoint. In the momentum equations ndim = 2 or 3
c                 whereas in the pressure and transport equations
c                 ndim = 1.
c     nprec   i   Number of diagonals which are used in the
c                 preconditioning.
c     prec    i   Array which contains the preconditioning matrix.
c     r       o   At output r is equal to A*usol. In the case of
c                 preconditioning A is the preconditioned matrix.
c     rhsd    i   Vector containing the right-hand side vector of the
c                 system of equations.
c     solutv  i   Work array for solut with virtual points
c     usol    o   Solution vector of length n. At output the array con-
c                 tains the last iterate, which is an approximation to the
c                 solution of the system.
c     work    o   Work array of size n * (itmax+4).
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
c     c       Arrays determining the plane rotations which transform the
c             upper Hessenberg-matrix into an upper triangular matrix.
c     hh      Array containing an upper Hessenberg-matrix of dimension
c             (i+1).
c     i       Loop variable for the inner iteration.
c     itmax   The maximum number of iterations to be performed.
c     j       Counting variables.
c     k       Counting variables.
c     r0norm  Norm of the initial residual vector.
c     rnorm   Norm of the residual vector.
c     rs      Array containing the norms of the residuals from the inner
c             iterations.
c     s       Arrays determining the plane rotations which transform the
c             upper Hessenberg-matrix into an upper triangular matrix.
c     t       Auxiliary variable.
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     daxpy   Computes a vector + a constant * other vector. This
c             is a BLAS routine.
c     ddot    Computes an inner product. This is a BLAS routine.
c     dinvl   Computes the multiplication of the inverse of a lower
c             triangular matrix, stored in arrays prec and amat,
c             and a vector.
c     dinvu   Computes the multiplication of the inverse of a upper
c             triangular matrix, stored in arrays prec and amat,
c             and a vector.
c     dnrm2   Calculates the 2-norm of a given vector. This is
c             a BLAS routine.
c     ismatvec   Computes the product of the sparse matrix, stored in AMAT,
c             and a vector.
c     prevc   Computes the product of the preconditioner, stored in
c             amat and prec.
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The subroutine polgmr is used to solve an unsymmetric system of
c     linear equations of the shape A.x = f. Starting from an initial
c     vector, in each iteration a vector is sought which minimizes the
c     residual vector A.x-f in a certain Krylov-subspace. The process
c     is used as a polynomial preconditioner in combination with the
c     GMRESR method. It uses the GMRES method with preconditioning.
c
c **change2** 28-06-1991 C. Vuik
c
c reason    : To calculate the residual with vector updates instead of
c             the formula r  = b-Ax
c                          k       k
c
c literature: Saad & Schultz p. 863
c
c **change2** end
c
c **change3**
c
c reason    :  To implement the Eisenstat trick
c
c References:  H. A. van der Vorst
c              Journal of Computational Physics 44 pp. 1-19 (1981)
c
c              S. C. Eisenstat
c              Research Report No. 185, Yale University, 1980
c
c **change3** end
c
c
c     Input:
c
c     The elements ICONTR(1), EPS,  ITMAX must have got
c     a value, and the arrays AMAT and RHSD  must have been filled by
c     the calling program.
c
c     Output:
c
c     The calculated solution vector is stored in USOL, and the
c     corresponding vector A*USOL in R.
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Parameters:

      integer n,imax,icontr(9), infmat(*),nprec,ndim,nconct, iinmtvc(*)
      double precision amat(n,*),rhsd(n),usol(n),r(n),eps,
     +                 work(n,*),prec(n,*),solutv(*)

c     --- Local parameters:

      integer          i,j,k,itmax
      double precision rnorm,t,hh(101,100),c(100),
     +                 s(100),rs(101),ddot,dnrm2,r0norm
      itmax = min(100,imax)
      do j=1,n
         work(j,1) = rhsd(j)
      end do
      rnorm = dnrm2(n,work,1)
      rs(1)=rnorm

c     --- change2**

      r0norm = rnorm

c     --- change2** end
c           Check for convergence.

      if (rnorm.lt.eps) then
         goto 500
      end if
      t = 1d0/rnorm
      do j = 1, n
         work(j,1) = work(j,1)*t
      end do

c     --- Main iteration loop.

      do i = 1, itmax

c     --- Inner iteration step, apply modified Gram-Schmid to form a
c         new column of hh.

         if(icontr(1).eq.2) then

c        --- change3**

            call dinvu(work(1,i),work(1,i+1),amat,n/ndim,ndim,
     +      nconct,prec,nprec,infmat)
            do 105 j = 1, n
               work(j,i+2) = (amat(j,1)-2d0)*work(j,i+1)
     +                       +work(j,i)
105         continue
            call dinvl(work(1,i+2),work(1,i+3),amat,n/ndim,ndim,
     +      nconct,prec,nprec,infmat)
            call daxpy(n,1d0,work(1,i+3),1,work(1,i+1),1)

c           --- change3** end

         else
            if(icontr(1).gt.0) then



              call ismatvec(amat,work(1,i),solutv,work(1,i+2),infmat,
     +                      icontr, iinmtvc)

              call prevc(n,work(1,i+2),work(1,i+1),amat,ndim,nconct,
     +                   prec,nprec,infmat,icontr(1))
            else
              call prevc(n,work(1,i),work(1,i+2),amat,ndim,nconct,
     +                   prec,nprec,infmat,icontr(1))



              call ismatvec(amat,work(1,i+2),solutv,work(1,i+1),infmat,
     +                      icontr, iinmtvc)

            end if
         end if
         do j=1,i
            t = ddot(n,work(1,j),1,work(1,i+1),1)
            hh(j,i)=t
            call daxpy(n,-t,work(1,j),1,work(1,i+1),1)
         end do
         hh(i+1,i) = dnrm2(n,work(1,i+1),1)

c        --- Perform plane rotations on new column.

         if (i.gt.1) then
            do k=2,i
               t=hh(k-1,i)
               hh(k-1,i) =  c(k-1)*t + s(k-1)*hh(k,i)
               hh(k,i)   = -s(k-1)*t + c(k-1)*hh(k,i)
            end do
         end if

c        --- Calculate new plane rotation.

         t = sqrt(hh(i,i)**2 + hh(i+1,i)**2)
         c(i) = hh(i,i)/t
         s(i) = hh(i+1,i)/t
         rs(i+1) = -s(i)*rs(i)
         rs(i) = c(i)*rs(i)
         hh(i,i) = c(i)*hh(i,i) + s(i)*hh(i+1,i)

c        --- Determine residual norm.

         rnorm = abs(rs(i+1))

c        --- Check for termination of inner iterations.

         if ( rnorm.lt.eps .or. i.eq.itmax) then

c        --- Compute new solution. First solve upper triangular system.

            rs(i) = rs(i)/hh(i,i)
            do k=i-1,1,-1
               t=rs(k)
               do j=k+1,i
                  t=t-hh(k,j)*rs(j)
               end do
               rs(k)=t/hh(k,k)
            end do

c           --- Form linear combinations of residuals to get solution.

            do j = 1, n
               usol(j) = rs(1)*work(j,1)
            end do
            do j=2,i
               call daxpy(n,rs(j),work(1,j),1,usol,1)
            end do

c           --- Now r = A*usol is computed.
c               change2**
c               compute R*y

            t = 1d0/hh(i+1,i)
            do j = 1, n
               work(j,i+1) = work(j,i+1)*t
            end do
            do j = 1, i
               rs(j) = hh(j,j)*rs(j)
               do k = j+1, i
                  rs(j) = rs(j)+hh(j,k)*rs(k)
               end do
            end do

c           --- T
c               compute Q *R*y

            rs(i+1) = 0d0
            do j = i+1, 2, -1
               t = rs(j-1)
               rs(j-1) = c(j-1)*t-s(j-1)*rs(j)
               rs(j)   = s(j-1)*t+c(j-1)*rs(j)
            end do

c           --- compute r

            do j = 1, n
               r(j) = rs(1)*work(j,1)
            end do
            do j = 2, i+1
               call daxpy(n,rs(j),work(1,j),1,r,1)
            end do

c           --- change2** end

            goto 500
         else
            t = 1d0/hh(i+1,i)
            do j = 1, n
               work(j,i+1) = work(j,i+1)*t
            end do
         end if
      end do
500   continue
      return
      end
